#include <stdio.h>
#include <stdlib.h>
#include <pam.h>
#include <gsl/gsl_matrix.h>
#include "libmat.h"

/* mat2pgm. This program takes a mat file from stdin, and writes a pgm
file to stdout.

The mat elements are quantized to the 0..255 range for a pgm file.

This program is for looking at a mat file contents through a picture viewer.

The maxvalue and minvalue of mat are printed in stderr.
*/

int main(int argn, char *argv[])
{
	struct pam out_pam;
	tuple **out_tuples;
	int row, column;

	float maxval, minval, scale, offset;

	gsl_matrix *m;

	pnm_init(&argn, argv);

	mat_fread(stdin, &m);

	/* Create the pam structure */
	out_pam.size = sizeof(out_pam);
	out_pam.len = out_pam.size;
	out_pam.file = stdout;
	out_pam.format = PGM_FORMAT;
	out_pam.plainformat = 0; /* False. It will be binary. */
	out_pam.height = m->size1;
	out_pam.width = m->size2;
	out_pam.depth = 1; /* 1 plane. Grayscale. */
	out_pam.maxval = 255;
	out_pam.bytes_per_sample = 1;
	/* Not needed out_pam.tuple_type = PAM_FORMAT_TYPE(PAM_PGM_TUPLETYPE);*/
	out_pam.allocation_depth = 1;

	maxval = gsl_matrix_max(m);
	minval = gsl_matrix_min(m);
	scale = maxval - minval;
	offset = -minval;

	fprintf(stderr,"Maxval: %g\nMinval: %g\n", maxval, minval);

	out_tuples = pnm_allocpamarray(&out_pam);
	if (out_tuples == NULL)
	{
		fprintf(stderr,"Error allocating the tuples.\n");
		return 1;
	}

	for (row = 0; row < m->size1; row++) {
		for (column = 0; column < m->size2; ++column) {
			/*
			fprintf(stderr,"%i ",
				(sample) ((m->data[row * m->tda + column] /
				scale + offset)
				* ((float) out_pam.maxval)));
				*/
			out_tuples[row][column][0] =
				(sample) ((m->data[row * m->tda + column] /
				scale + offset)
				* ((float) out_pam.maxval));
		}
		/*
		fprintf(stderr,"\n");
		*/
	}

	/* Write the pgm file (to stdout) */
	pnm_writepam(&out_pam, out_tuples);

	/* Free all the memory */
	pnm_freepamarray(out_tuples, &out_pam);
	gsl_matrix_free(m);

	return 0;
}
